/*
 * Copyright (C) 2006 uguu@users.sourceforge.jp, All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    1. Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *    2. Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *
 *    3. Neither the name of Clarkware Consulting, Inc. nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without prior written permission. For written
 *       permission, please contact clarkware@clarkware.com.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * CLARKWARE CONSULTING OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN  ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package jp.sourceforge.rpn_computer;

import java.util.Stack;

/**
 * <p>
 * 計算中の状態を保持するコンテキストです。
 * </p>
 * 
 * @author uguu@users.sourceforge.jp
 */
public final class RpnContext {

    private Stack stack = new Stack();

    /**
     * <p>
     * スタックに格納されている要素の数を返します。
     * </p>
     * 
     * @return スタックに格納されている要素の数。
     */
    public int sizeStack() {
        return this.stack.size();
    }

    /**
     * <p>
     * スタックの先頭から値を取り出します。取り出した値は、スタックから削除されます。スタックが空の場合、{@link ComputeException}例外をスローします。
     * </p>
     * 
     * @return スタックから取り出した値。
     */
    public Double popStack() {
        if (this.stack.empty()) {
            throw new ComputeException("スタックが空です。");
        }
        return (Double) this.stack.pop();
    }

    /**
     * <p>
     * スタックの先頭から値を取り出します。取り出した値は、スタックの先頭から削除されません。スタックが空の場合、{@link ComputeException}例外をスローします。
     * </p>
     * 
     * @return スタックから取り出した値。
     */
    public Double peekStack() {
        if (this.stack.empty()) {
            throw new ComputeException("スタックが空です。");
        }
        return (Double) this.stack.peek();
    }

    /**
     * <p>
     * スタックの先頭に値を追加します。
     * </p>
     * 
     * @param value
     *            スタックに追加する値。nullの場合、{@link ComputeException}例外をスローします。
     */
    public void pushStack(Double value) {
        if (value == null) {
            throw new ComputeException("スタックに追加しようとした値がnullです。");
        }
        this.stack.push(value);
    }
}
