/*
 * Copyright (C) 2006 uguu@users.sourceforge.jp, All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    1. Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *    2. Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *
 *    3. Neither the name of Clarkware Consulting, Inc. nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without prior written permission. For written
 *       permission, please contact clarkware@clarkware.com.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * CLARKWARE CONSULTING OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN  ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package jp.sourceforge.rpn_computer.node;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import jp.sourceforge.rpn_computer.RpnNode;

/**
 * <p>
 * ノードの基底抽象クラスです。
 * </p>
 * 
 * @author uguu@users.sourceforge.jp
 */
public abstract class AbstractNode implements RpnNode {

    private List nodeList = new ArrayList();

    /**
     * <p>
     * 自分の子ノードを追加します。
     * </p>
     * 
     * @param node
     *            子ノード。nullの場合、{@link NullPointerException}例外をスローします。
     */
    protected final void addNode(RpnNode node) {
        if (node == null) {
            throw new NullPointerException("nodeがnullです。");
        }
        this.nodeList.add(node);
    }

    /**
     * {@inheritDoc}
     */
    public final RpnNode[] getChildren() {
        return (RpnNode[]) this.nodeList.toArray(new RpnNode[0]);
    }

    /**
     * <p>
     * 子ノードの文字列表現を返します。
     * </p>
     * 
     * @return 子ノードの文字列表現。
     */
    protected final String toChildrenString() {
        String str = "";
        for (Iterator i = this.nodeList.iterator(); i.hasNext();) {
            RpnNode node = (RpnNode) i.next();
            String[] nodeString = node.toString().split("\n");
            for (int j = 0; j < nodeString.length; j++) {
                str += "\t" + nodeString[j] + "\n";
            }
        }
        return str;
    }
}
